package knightminer.inspirations.library.recipe.cauldron.contenttype;

import knightminer.inspirations.library.recipe.cauldron.contents.ICauldronContents;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.registries.IForgeRegistry;
import net.minecraftforge.registries.IForgeRegistryEntry;

import javax.annotation.Nullable;
import java.util.Objects;
import java.util.function.Function;

/**
 * Content type that mirrors a Forge registry
 * @param <T>  Value type
 * @param <C>  Cauldron content type
 */
public class RegistryContentType<C extends ICauldronContents, T extends IForgeRegistryEntry<T>> extends CauldronContentType<C, T> {
  private final IForgeRegistry<T> registry;

  /**
   * Creates a new instance
   * @param clazz        Class for validation
   * @param constructor  Class constructor
   * @param registry     Forge registry instance for lookups
   * @param valueGetter  Function to get the value from a content type
   */
  public RegistryContentType(Class<C> clazz, Function<? super T,? extends C> constructor, IForgeRegistry<T> registry, Function<C,T> valueGetter) {
    super(clazz, constructor, valueGetter);
    this.registry = registry;
  }

  @Override
  public String getName(T value) {
    return Objects.requireNonNull(value.getRegistryName()).toString();
  }

  @Nullable
  @Override
  public T getEntry(String name) {
    ResourceLocation location = ResourceLocation.tryCreate(name);
    if (location != null) {
      // swap default value for null
      T value = registry.getValue(location);
      ResourceLocation defaultKey = registry.getDefaultKey();
      if (value != null && defaultKey != null && !defaultKey.equals(value.getRegistryName())) {
        return value;
      }
    }
    return null;
  }
}
